/* pz.h ... an archiver for Palm OS */

/*
 * constants
 */

#define DEFAULT_CARD_NO 0

#define PZ_CREATOR 'PZIP'
#define PZ_DB_TYPE 'PZDB'
#define PZ_DB_EXT ".pz"
#define PZ_DB_EXT_LENGTH 3

#define noErr 0
#define ERROR -1
#define DB_ERROR -2
#define DATA_ERROR -3
#define MEMORY_ERROR -4
#define STORAGE_ERROR -5
#define ZLIB_ERROR -6
#define CANCELED 1

#define DATA_DB_EXT ".pdb"
#define RES_DB_EXT ".prc"
#define DB_EXT_LENGTH 4

#define PZ_ID1 0x1f			/* gzip ID1 */
#define PZ_ID2 ((Byte) 0x8b)		/* gzip ID2 */
#define PZ_CM_STORED 0			/* store method */
#define PZ_CM_LZHED 3			/* lzh method */
#define PZ_CM_IMPLODED 6		/* implode method */
#define PZ_CM_DEFLATED 8		/* deflate method */
#define PZ_FLG_FTEXT 1			/* gzip FTEXT flag */
#define PZ_FLG_FHCRC 2			/* gzip FHCRC flag */
#define PZ_FLG_FEXTRA 4			/* gzip FEXTRA flag */
#define PZ_FLG_FNAME 8			/* gzip FNAME flag */
#define PZ_FLG_FCOMMENT 16		/* gzip FCOMMENT flag */
#define PZ_OS_UNKNOWN ((Byte) 0xff)	/* Palm OS is not defined */

#define PZ_XLEN_SIZE 2
#define PZ_CRC16_SIZE 2

#define STREAM_TRAILER_SIZE 4

#define STREAM_FLAG_UNKNOWN 0
#define STREAM_FLAG_VALID 1

#define UPDATE_DB_LIST 1
#define UPDATE_DB_SORT 2
#define UPDATE_DB_FILTER 4

#define UPDATE_ARCHIVE 1


/*
 * types
 */

enum db_where {
    where_all,
    where_ram,
    where_rom,
    where_flash,
    where_other,
    where_max
};

enum db_kind {
    kind_all,
    kind_system,
    kind_archive,
    kind_application,
    kind_panel,
    kind_hack,
    kind_da,
    kind_localizer,
    kind_data,
    kind_doc,
    kind_image,
    kind_unknown,
    kind_max
};

enum db_sort_by {
    sort_by_name,
    sort_by_where,
    sort_by_kind,
    sort_by_type,
    sort_by_creator,
    sort_by_recs,
    sort_by_size,
    sort_by_attr,
    sort_by_cdat,
    sort_by_mdat,
    sort_by_bdat,
    sort_by_vers,
    sort_by_mods,
    sort_by_none,
    sort_by_max
};

enum db_info0 {
    info0_none,
    info0_where,
    info0_kind,
    info0_max
};

enum db_info {
    info_none,
    info_type,
    info_creator,
    info_recs,
    info_size,
    info_attr,
    info_cdat,
    info_mdat,
    info_bdat,
    info_vers,
    info_mods,
    info_max
};

struct db_spec {
    UInt card_no;
    LocalID db_id;
};

struct db_rec {
    enum db_where where;
    enum db_kind kind;
    struct db_spec spec;
    ULong recs;
    ULong size;
};

struct pz_header {
    Byte id1;
    Byte id2;
    Byte cm;
    Byte flg;
    DWord mtime;
    Byte xfl;
    Byte os;
};

struct stream_info {
    UInt index;
    UInt num_records;
    ULong size;
    ULong uncompressed;
    Byte method;
    Byte flag;
    UInt name_offset;
    UInt name_length;
    UInt data_offset;
};

struct stream {
    DmOpenRef db;
    UInt first_index;
    UInt num_records;

    UInt cur_index;
    VoidHand cur_handle;
    VoidPtr cur_ptr;

    ULong cur_start;
    ULong cur_size;
    ULong cur_offset;
};

typedef int (*progress_callback_t)(ULong val, ULong max);


/*
 * global variables
 */

struct pz_old_prefs {
#ifndef UNPZ
    enum db_where where;
    enum db_kind kind;

    enum db_sort_by sort_by;

    enum db_info0 info0;
    enum db_info info1;
    enum db_info info2;
#endif

    struct db_spec last_selection;
    struct db_spec last_archive;

    Word last_form_id;
};

extern struct pz_prefs {
#ifndef UNPZ
    enum db_where where;
    enum db_kind kind;

    enum db_sort_by sort_by;

    enum db_info0 info0;
    enum db_info info1;
    enum db_info info2;
#endif

    struct db_spec last_selection;
#ifndef UNPZ
    struct db_spec last_archive;
#endif

    Word last_form_id;
    int last_db_top;
} prefs;

extern struct db_rec *db_list;
extern int num_dbs;


/*
 * function prototypes
 */

/* pz.c */

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags);

/* db_form.c */

Boolean handle_db_form_event(EventPtr event);

/* db_list.c */

void gen_db_list(enum db_where where, enum db_kind kind);
void free_db_list(void);

void sort_db_list(enum db_sort_by sort_by);

int find_db(char chr);

void draw_chars(const char *str, int length, int x, RectanglePtr bounds);
void draw_fit_chars(
	const char *str, int x, int width, RectanglePtr bounds, Boolean center);
void print_size(char *str, ULong size);
void draw_db_item(VoidPtr table, Word row, Word column, RectanglePtr bounds);

void draw_free_memory(Word gadget_id);

/* arch_form.c */

Boolean open_arch_db(UInt card_no, LocalID db_id);
Boolean handle_arch_form_event(EventPtr event);

/* db_info.c */

Boolean get_db_info(UInt card_no, LocalID db_id);
void set_db_info(void *db_header, ULong size, ULong compressed);

void print_ratio(char *str, ULong size, ULong compressed);

Boolean handle_info_form_event(EventPtr event);

/* read_form.c */

void open_read_form(DmOpenRef arch_db, struct stream_info *stream);
Boolean handle_read_form_event(EventPtr event);

/* progress.c */

int show_progress(ULong val, ULong max);

FormPtr start_progress(FormPtr *saved_form, CharPtr title, CharPtr name);
void end_progress(FormPtr form, FormPtr saved_form);

/* table.c */

void init_table(
        int table_id, int scrollbar_id, TableDrawItemFuncPtr draw_procedure);
void set_table_items(int items, int top, int selection);

void redraw_table(Boolean update_scrollbar);
void scroll_table(int delta, Boolean update_scrollbar);

int get_table_top(void);
int get_table_selection(void);

Boolean handle_table_event(EventPtr event, void (*double_func)(void));

/* util.c */

VoidPtr GetObjectPtrIn(FormPtr form, Word id);
VoidPtr GetObjectPtr(Word id);
void GetObjectBounds(Word id, RectanglePtr r);

void DrawBitmapOver(Word bitmap_id, Word object_id);
void SetPopup(Word popup_id, Word list_id, Word selection);

/* stream.c */

int count_streams(DmOpenRef db, struct stream_info **streams, int *bad);
int remove_stream(DmOpenRef db, UInt index, UInt num_records);

void print_stream_name(
	DmOpenRef db, struct stream_info *stream, char *name, int length);

Byte *open_in_stream(
	struct stream *in, DmOpenRef db, struct stream_info *stream);
void close_in_stream(struct stream *in);
ULong avail_in_stream(struct stream *in, ULong total_in, Byte **next_in);

int open_out_stream(
	struct stream *out, DmOpenRef db, UInt index, ULong max_size);
void close_out_stream(struct stream *out, ULong trailer);
int write_out_stream(struct stream *out, const void *ptr, ULong size);

/* compress_db.c */

int compress_db(DmOpenRef arch_db, UInt index, UInt card_no, LocalID db_id,
	ULong *uncompr_size, ULong *compr_size, progress_callback_t callback);

/* uncompress_db.c */

int uncompress_db(DmOpenRef arch_db, struct stream_info *stream,
	UInt card_no, progress_callback_t callback);

int uncompress_bytes(DmOpenRef arch_db, struct stream_info *stream,
	void *ptr, ULong size, ULong skip_size, int skip_count);

#define uncompress_header(arch_db, stream, ptr) \
	uncompress_bytes(arch_db, stream, ptr, \
		(ULong) &((DatabaseHdrPtr) 0)->recordList.firstEntry, 0, 0)
