/* read_form.c */

#include <Pilot.h>

#include "callback.h"

#include "pz.h"
#include "pz_rcp.h"


/*
 * constants
 */

#define PAGE_SIZE 4096


/*
 * static variables
 */

static DmOpenRef arch_db;
static struct stream_info *stream;

static int cur_page;
static int max_page;

static char title[dmDBNameLength];
static char page_str[12];


/*
 * entry function
 */

void open_read_form(DmOpenRef db, struct stream_info *s)
{
    arch_db = db;
    stream = s;
    cur_page = 0;
    if (stream->uncompressed > PAGE_SIZE) {
	max_page = (stream->uncompressed - 1) / PAGE_SIZE;
    } else {
	max_page = 0;
    }

    WinEraseWindow();
    FrmPopupForm(READ_FORM_ID);
}


/*
 * text handling
 */

static void convert_newlines(char *text)
{
    char *p, *q, c;

    p = text;
    q = text;
    do {
	c = *p++;
	if (c == '\r') {
	    if (*p == '\n') {
		++p;
	    }
	    c = '\n';
	}
	*q++ = c;
    } while (c != '\0');
}

static Handle get_contents(int page)
{
    ULong size;
    VoidHand h;
    CharPtr p;
    int err;

    if (max_page == 0) {
	size = stream->uncompressed;
	h = MemHandleNew(size + 1);
    } else {
	if (page == max_page) {
	    size = stream->uncompressed - PAGE_SIZE * page;
	} else {
	    size = PAGE_SIZE;
	}
	h = MemHandleNew(PAGE_SIZE + 1);
    }

    if (h != NULL) {
	p = MemHandleLock(h);
	err = uncompress_bytes(arch_db, stream, p, size, PAGE_SIZE, page);
	if (err == noErr) {
	    p[size] = '\0';
	    convert_newlines(p);
	    MemHandleUnlock(h);
	} else {
	    MemHandleUnlock(h);
	    MemHandleFree(h);
	    h = NULL;
	}
    }

    return (Handle) h;
}


/*
 * event handlers
 */

static void set_scrollbar(FormPtr form, FieldPtr field)
{
    ScrollBarPtr scrollbar;
    Word scroll_pos, text_height, field_height, max_value;

    scrollbar = GetObjectPtrIn(form, READ_SCROLLBAR_ID);

    FldGetScrollValues(field, &scroll_pos, &text_height, &field_height);
    if (text_height > field_height) {
	max_value = text_height - field_height;
    } else if (scroll_pos > 0) {
	max_value = scroll_pos;
    } else {
	max_value = 0;
    }
    SclSetScrollBar(scrollbar, scroll_pos, 0, max_value, field_height);
}

static void scroll_field(int delta, Boolean update_scrollbar)
{
    FieldPtr field;

    field = GetObjectPtr(READ_FIELD_ID);

    if (delta < 0) {
	FldScrollField(field, - delta, up);
    } else {
	FldScrollField(field, delta, down);
    }

    if (update_scrollbar) {
	set_scrollbar(FrmGetActiveForm(), field);
    }
}

static void set_page(int page)
{
    FieldPtr field, page_field;
    Handle h;

    if (page >= 0 && page <= max_page) {
	cur_page = page;

	DrawBitmapOver(WATCH_ICON_ID, READ_FIELD_ID);

	field = GetObjectPtr(READ_FIELD_ID);
	FldFreeMemory(field);
	h = get_contents(cur_page);

	page_field = GetObjectPtr(READ_PAGE_ID);
	StrPrintF(page_str, "%d/%d", cur_page + 1, max_page + 1);
	FldSetTextPtr(page_field, page_str);
	FldDrawField(page_field);

	if (h != NULL) {
	    FldSetTextHandle(field, h);
	    FldDrawField(field);
	} else {
	    FldSetTextPtr(field, "decompression failed.");
	    FldRecalculateField(field, true);
	}
	set_scrollbar(FrmGetActiveForm(), field);
    }
}

Boolean handle_read_form_event(EventPtr event)
{
    FormPtr form;
    Boolean handled = false;

    CALLBACK_PROLOGUE

    switch (event->eType) {
    case frmOpenEvent:
	form = FrmGetActiveForm();
	print_stream_name(arch_db, stream, title, dmDBNameLength);
	FrmSetTitle(form, title);
	if (max_page > 0) {
	    FrmShowObject(form, FrmGetObjectIndex(form, READ_PREV_ID));
	    FrmShowObject(form, FrmGetObjectIndex(form, READ_NEXT_ID));
	}
	FrmDrawForm(form);
	set_page(0);
	handled = true;
	break;

    case ctlSelectEvent:
	switch (event->data.ctlSelect.controlID) {
	case READ_COPY_ID:
	    FldCopy(GetObjectPtr(READ_FIELD_ID));

	    /* fall through */

	case READ_DONE_ID:
	    FrmReturnToForm(prefs.last_form_id);
	    FrmUpdateForm(prefs.last_form_id, 0);
	    handled = true;
	    break;

	case READ_PREV_ID:
	    set_page(cur_page - 1);
	    handled = true;
	    break;

	case READ_NEXT_ID:
	    set_page(cur_page + 1);
	    handled = true;
	    break;
	}
	break;

    case keyDownEvent:
	if (event->data.keyDown.chr == pageUpChr) {
	    scroll_field(- READ_FIELD_LINES + 1, true);
	    handled = true;
	} else if (event->data.keyDown.chr == pageDownChr) {
	    scroll_field(READ_FIELD_LINES - 1, true);
	    handled = true;
	}
	break;

    case fldChangedEvent:
	scroll_field(0, true);
	handled = true;
	break;

    case sclRepeatEvent:
	scroll_field(
		event->data.sclRepeat.newValue - event->data.sclRepeat.value,
		false);
	break;

    default:
	break;
    }

    CALLBACK_EPILOGUE

    return handled;
}
